@@ -1,36 +1,37 @@
-# -*- coding: utf-8 -*-
 """
 Maximum flow (and minimum cut) algorithms on capacitated graphs.
 """
+
 import networkx as nx
 
-# Define the default flow function for computing maximum flow.
+from .boykovkolmogorov import boykov_kolmogorov
+from .dinitz_alg import dinitz
 from .edmondskarp import edmonds_karp
 from .preflowpush import preflow_push
 from .shortestaugmentingpath import shortest_augmenting_path
 from .utils import build_flow_dict
+
+# Define the default flow function for computing maximum flow.
 default_flow_func = preflow_push
 
-__all__ = ['maximum_flow',
-           'maximum_flow_value',
-           'minimum_cut',
-           'minimum_cut_value']
+__all__ = ["maximum_flow", "maximum_flow_value", "minimum_cut", "minimum_cut_value"]
 
 
-def maximum_flow(G, s, t, capacity='capacity', flow_func=None, **kwargs):
+@nx._dispatchable(graphs="flowG", edge_attrs={"capacity": float("inf")})
+def maximum_flow(flowG, _s, _t, capacity="capacity", flow_func=None, **kwargs):
     """Find a maximum single-commodity flow.
 
     Parameters
     ----------
-    G : NetworkX graph
+    flowG : NetworkX graph
         Edges of the graph are expected to have an attribute called
         'capacity'. If this attribute is not present, the edge is
         considered to have infinite capacity.
 
-    s : node
+    _s : node
         Source node for the flow.
 
-    t : node
+    _t : node
         Sink node for the flow.
 
     capacity : string
@@ -85,7 +86,7 @@ def maximum_flow(G, s, t, capacity='capacity', flow_func=None, **kwargs):
 
     Notes
     -----
-    The function used in the flow_func paramter has to return a residual
+    The function used in the flow_func parameter has to return a residual
     network that follows NetworkX conventions:
 
     The residual network :samp:`R` from an input graph :samp:`G` has the
@@ -117,64 +118,67 @@ def maximum_flow(G, s, t, capacity='capacity', flow_func=None, **kwargs):
 
     Examples
     --------
-    >>> import networkx as nx
     >>> G = nx.DiGraph()
-    >>> G.add_edge('x','a', capacity=3.0)
-    >>> G.add_edge('x','b', capacity=1.0)
-    >>> G.add_edge('a','c', capacity=3.0)
-    >>> G.add_edge('b','c', capacity=5.0)
-    >>> G.add_edge('b','d', capacity=4.0)
-    >>> G.add_edge('d','e', capacity=2.0)
-    >>> G.add_edge('c','y', capacity=2.0)
-    >>> G.add_edge('e','y', capacity=3.0)
+    >>> G.add_edge("x", "a", capacity=3.0)
+    >>> G.add_edge("x", "b", capacity=1.0)
+    >>> G.add_edge("a", "c", capacity=3.0)
+    >>> G.add_edge("b", "c", capacity=5.0)
+    >>> G.add_edge("b", "d", capacity=4.0)
+    >>> G.add_edge("d", "e", capacity=2.0)
+    >>> G.add_edge("c", "y", capacity=2.0)
+    >>> G.add_edge("e", "y", capacity=3.0)
 
     maximum_flow returns both the value of the maximum flow and a
     dictionary with all flows.
 
-    >>> flow_value, flow_dict = nx.maximum_flow(G, 'x', 'y')
+    >>> flow_value, flow_dict = nx.maximum_flow(G, "x", "y")
     >>> flow_value
     3.0
-    >>> print(flow_dict['x']['b'])
+    >>> print(flow_dict["x"]["b"])
     1.0
 
     You can also use alternative algorithms for computing the
     maximum flow by using the flow_func parameter.
 
     >>> from networkx.algorithms.flow import shortest_augmenting_path
-    >>> flow_value == nx.maximum_flow(G, 'x', 'y',
-    ...                         flow_func=shortest_augmenting_path)[0]
+    >>> flow_value == nx.maximum_flow(G, "x", "y", flow_func=shortest_augmenting_path)[
+    ...     0
+    ... ]
     True
 
     """
     if flow_func is None:
         if kwargs:
-            raise nx.NetworkXError("You have to explicitly set a flow_func if"
-                                   " you need to pass parameters via kwargs.")
+            raise nx.NetworkXError(
+                "You have to explicitly set a flow_func if"
+                " you need to pass parameters via kwargs."
+            )
         flow_func = default_flow_func
 
     if not callable(flow_func):
         raise nx.NetworkXError("flow_func has to be callable.")
 
-    R = flow_func(G, s, t, capacity=capacity, value_only=False, **kwargs)
-    flow_dict = build_flow_dict(G, R)
+    R = flow_func(flowG, _s, _t, capacity=capacity, value_only=False, **kwargs)
+    flow_dict = build_flow_dict(flowG, R)
 
-    return (R.graph['flow_value'], flow_dict)
+    return (R.graph["flow_value"], flow_dict)
 
 
-def maximum_flow_value(G, s, t, capacity='capacity', flow_func=None, **kwargs):
+@nx._dispatchable(graphs="flowG", edge_attrs={"capacity": float("inf")})
+def maximum_flow_value(flowG, _s, _t, capacity="capacity", flow_func=None, **kwargs):
     """Find the value of maximum single-commodity flow.
 
     Parameters
     ----------
-    G : NetworkX graph
+    flowG : NetworkX graph
         Edges of the graph are expected to have an attribute called
         'capacity'. If this attribute is not present, the edge is
         considered to have infinite capacity.
 
-    s : node
+    _s : node
         Source node for the flow.
 
-    t : node
+    _t : node
         Sink node for the flow.
 
     capacity : string
@@ -225,7 +229,7 @@ def maximum_flow_value(G, s, t, capacity='capacity', flow_func=None, **kwargs):
 
     Notes
     -----
-    The function used in the flow_func paramter has to return a residual
+    The function used in the flow_func parameter has to return a residual
     network that follows NetworkX conventions:
 
     The residual network :samp:`R` from an input graph :samp:`G` has the
@@ -257,21 +261,20 @@ def maximum_flow_value(G, s, t, capacity='capacity', flow_func=None, **kwargs):
 
     Examples
     --------
-    >>> import networkx as nx
     >>> G = nx.DiGraph()
-    >>> G.add_edge('x','a', capacity=3.0)
-    >>> G.add_edge('x','b', capacity=1.0)
-    >>> G.add_edge('a','c', capacity=3.0)
-    >>> G.add_edge('b','c', capacity=5.0)
-    >>> G.add_edge('b','d', capacity=4.0)
-    >>> G.add_edge('d','e', capacity=2.0)
-    >>> G.add_edge('c','y', capacity=2.0)
-    >>> G.add_edge('e','y', capacity=3.0)
+    >>> G.add_edge("x", "a", capacity=3.0)
+    >>> G.add_edge("x", "b", capacity=1.0)
+    >>> G.add_edge("a", "c", capacity=3.0)
+    >>> G.add_edge("b", "c", capacity=5.0)
+    >>> G.add_edge("b", "d", capacity=4.0)
+    >>> G.add_edge("d", "e", capacity=2.0)
+    >>> G.add_edge("c", "y", capacity=2.0)
+    >>> G.add_edge("e", "y", capacity=3.0)
 
     maximum_flow_value computes only the value of the
     maximum flow:
 
-    >>> flow_value = nx.maximum_flow_value(G, 'x', 'y')
+    >>> flow_value = nx.maximum_flow_value(G, "x", "y")
     >>> flow_value
     3.0
 
@@ -279,26 +282,30 @@ def maximum_flow_value(G, s, t, capacity='capacity', flow_func=None, **kwargs):
     maximum flow by using the flow_func parameter.
 
     >>> from networkx.algorithms.flow import shortest_augmenting_path
-    >>> flow_value == nx.maximum_flow_value(G, 'x', 'y',
-    ...                                     flow_func=shortest_augmenting_path)
+    >>> flow_value == nx.maximum_flow_value(
+    ...     G, "x", "y", flow_func=shortest_augmenting_path
+    ... )
     True
 
     """
     if flow_func is None:
         if kwargs:
-            raise nx.NetworkXError("You have to explicitly set a flow_func if"
-                                   " you need to pass parameters via kwargs.")
+            raise nx.NetworkXError(
+                "You have to explicitly set a flow_func if"
+                " you need to pass parameters via kwargs."
+            )
         flow_func = default_flow_func
 
     if not callable(flow_func):
         raise nx.NetworkXError("flow_func has to be callable.")
 
-    R = flow_func(G, s, t, capacity=capacity, value_only=True, **kwargs)
+    R = flow_func(flowG, _s, _t, capacity=capacity, value_only=True, **kwargs)
 
-    return R.graph['flow_value']
+    return R.graph["flow_value"]
 
 
-def minimum_cut(G, s, t, capacity='capacity', flow_func=None, **kwargs):
+@nx._dispatchable(graphs="flowG", edge_attrs={"capacity": float("inf")})
+def minimum_cut(flowG, _s, _t, capacity="capacity", flow_func=None, **kwargs):
     """Compute the value and the node partition of a minimum (s, t)-cut.
 
     Use the max-flow min-cut theorem, i.e., the capacity of a minimum
@@ -306,15 +313,15 @@ def minimum_cut(G, s, t, capacity='capacity', flow_func=None, **kwargs):
 
     Parameters
     ----------
-    G : NetworkX graph
+    flowG : NetworkX graph
         Edges of the graph are expected to have an attribute called
         'capacity'. If this attribute is not present, the edge is
         considered to have infinite capacity.
 
-    s : node
+    _s : node
         Source node for the flow.
 
-    t : node
+    _t : node
         Sink node for the flow.
 
     capacity : string
@@ -362,7 +369,7 @@ def minimum_cut(G, s, t, capacity='capacity', flow_func=None, **kwargs):
 
     Notes
     -----
-    The function used in the flow_func paramter has to return a residual
+    The function used in the flow_func parameter has to return a residual
     network that follows NetworkX conventions:
 
     The residual network :samp:`R` from an input graph :samp:`G` has the
@@ -394,21 +401,20 @@ def minimum_cut(G, s, t, capacity='capacity', flow_func=None, **kwargs):
 
     Examples
     --------
-    >>> import networkx as nx
     >>> G = nx.DiGraph()
-    >>> G.add_edge('x','a', capacity = 3.0)
-    >>> G.add_edge('x','b', capacity = 1.0)
-    >>> G.add_edge('a','c', capacity = 3.0)
-    >>> G.add_edge('b','c', capacity = 5.0)
-    >>> G.add_edge('b','d', capacity = 4.0)
-    >>> G.add_edge('d','e', capacity = 2.0)
-    >>> G.add_edge('c','y', capacity = 2.0)
-    >>> G.add_edge('e','y', capacity = 3.0)
+    >>> G.add_edge("x", "a", capacity=3.0)
+    >>> G.add_edge("x", "b", capacity=1.0)
+    >>> G.add_edge("a", "c", capacity=3.0)
+    >>> G.add_edge("b", "c", capacity=5.0)
+    >>> G.add_edge("b", "d", capacity=4.0)
+    >>> G.add_edge("d", "e", capacity=2.0)
+    >>> G.add_edge("c", "y", capacity=2.0)
+    >>> G.add_edge("e", "y", capacity=3.0)
 
     minimum_cut computes both the value of the
     minimum cut and the node partition:
 
-    >>> cut_value, partition = nx.minimum_cut(G, 'x', 'y')
+    >>> cut_value, partition = nx.minimum_cut(G, "x", "y")
     >>> reachable, non_reachable = partition
 
     'partition' here is a tuple with the two sets of nodes that define
@@ -420,50 +426,49 @@ def minimum_cut(G, s, t, capacity='capacity', flow_func=None, **kwargs):
     ...     cutset.update((u, v) for v in nbrs if v in non_reachable)
     >>> print(sorted(cutset))
     [('c', 'y'), ('x', 'b')]
-    >>> cut_value == sum(G.edge[u][v]['capacity'] for (u, v) in cutset)
+    >>> cut_value == sum(G.edges[u, v]["capacity"] for (u, v) in cutset)
     True
 
     You can also use alternative algorithms for computing the
     minimum cut by using the flow_func parameter.
 
     >>> from networkx.algorithms.flow import shortest_augmenting_path
-    >>> cut_value == nx.minimum_cut(G, 'x', 'y',
-    ...                             flow_func=shortest_augmenting_path)[0]
+    >>> cut_value == nx.minimum_cut(G, "x", "y", flow_func=shortest_augmenting_path)[0]
     True
 
     """
     if flow_func is None:
         if kwargs:
-            raise nx.NetworkXError("You have to explicitly set a flow_func if"
-                                   " you need to pass parameters via kwargs.")
+            raise nx.NetworkXError(
+                "You have to explicitly set a flow_func if"
+                " you need to pass parameters via kwargs."
+            )
         flow_func = default_flow_func
 
     if not callable(flow_func):
         raise nx.NetworkXError("flow_func has to be callable.")
 
-    if (kwargs.get('cutoff') is not None and
-        flow_func in (edmonds_karp, preflow_push, shortest_augmenting_path)):
+    if kwargs.get("cutoff") is not None and flow_func is preflow_push:
         raise nx.NetworkXError("cutoff should not be specified.")
 
-    R = flow_func(G, s, t, capacity=capacity, value_only=True, **kwargs)
-    # Remove saturated edges from the residual network 
-    cutset = [(u, v, d) for u, v, d in R.edges(data=True)
-              if d['flow'] == d['capacity']]
+    R = flow_func(flowG, _s, _t, capacity=capacity, value_only=True, **kwargs)
+    # Remove saturated edges from the residual network
+    cutset = [(u, v, d) for u, v, d in R.edges(data=True) if d["flow"] == d["capacity"]]
     R.remove_edges_from(cutset)
 
     # Then, reachable and non reachable nodes from source in the
     # residual network form the node partition that defines
     # the minimum cut.
-    non_reachable = set(nx.shortest_path_length(R, target=t))
-    partition = (set(G) - non_reachable, non_reachable)
-    # Finaly add again cutset edges to the residual network to make
+    non_reachable = set(dict(nx.shortest_path_length(R, target=_t)))
+    partition = (set(flowG) - non_reachable, non_reachable)
+    # Finally add again cutset edges to the residual network to make
     # sure that it is reusable.
-    if cutset is not None:
-        R.add_edges_from(cutset)
-    return (R.graph['flow_value'], partition)
+    R.add_edges_from(cutset)
+    return (R.graph["flow_value"], partition)
 
 
-def minimum_cut_value(G, s, t, capacity='capacity', flow_func=None, **kwargs):
+@nx._dispatchable(graphs="flowG", edge_attrs={"capacity": float("inf")})
+def minimum_cut_value(flowG, _s, _t, capacity="capacity", flow_func=None, **kwargs):
     """Compute the value of a minimum (s, t)-cut.
 
     Use the max-flow min-cut theorem, i.e., the capacity of a minimum
@@ -471,15 +476,15 @@ def minimum_cut_value(G, s, t, capacity='capacity', flow_func=None, **kwargs):
 
     Parameters
     ----------
-    G : NetworkX graph
+    flowG : NetworkX graph
         Edges of the graph are expected to have an attribute called
         'capacity'. If this attribute is not present, the edge is
         considered to have infinite capacity.
 
-    s : node
+    _s : node
         Source node for the flow.
 
-    t : node
+    _t : node
         Sink node for the flow.
 
     capacity : string
@@ -524,7 +529,7 @@ def minimum_cut_value(G, s, t, capacity='capacity', flow_func=None, **kwargs):
 
     Notes
     -----
-    The function used in the flow_func paramter has to return a residual
+    The function used in the flow_func parameter has to return a residual
     network that follows NetworkX conventions:
 
     The residual network :samp:`R` from an input graph :samp:`G` has the
@@ -556,21 +561,20 @@ def minimum_cut_value(G, s, t, capacity='capacity', flow_func=None, **kwargs):
 
     Examples
     --------
-    >>> import networkx as nx
     >>> G = nx.DiGraph()
-    >>> G.add_edge('x','a', capacity = 3.0)
-    >>> G.add_edge('x','b', capacity = 1.0)
-    >>> G.add_edge('a','c', capacity = 3.0)
-    >>> G.add_edge('b','c', capacity = 5.0)
-    >>> G.add_edge('b','d', capacity = 4.0)
-    >>> G.add_edge('d','e', capacity = 2.0)
-    >>> G.add_edge('c','y', capacity = 2.0)
-    >>> G.add_edge('e','y', capacity = 3.0)
+    >>> G.add_edge("x", "a", capacity=3.0)
+    >>> G.add_edge("x", "b", capacity=1.0)
+    >>> G.add_edge("a", "c", capacity=3.0)
+    >>> G.add_edge("b", "c", capacity=5.0)
+    >>> G.add_edge("b", "d", capacity=4.0)
+    >>> G.add_edge("d", "e", capacity=2.0)
+    >>> G.add_edge("c", "y", capacity=2.0)
+    >>> G.add_edge("e", "y", capacity=3.0)
 
     minimum_cut_value computes only the value of the
     minimum cut:
 
-    >>> cut_value = nx.minimum_cut_value(G, 'x', 'y')
+    >>> cut_value = nx.minimum_cut_value(G, "x", "y")
     >>> cut_value
     3.0
 
@@ -578,24 +582,26 @@ def minimum_cut_value(G, s, t, capacity='capacity', flow_func=None, **kwargs):
     minimum cut by using the flow_func parameter.
 
     >>> from networkx.algorithms.flow import shortest_augmenting_path
-    >>> cut_value == nx.minimum_cut_value(G, 'x', 'y',
-    ...                                   flow_func=shortest_augmenting_path)
+    >>> cut_value == nx.minimum_cut_value(
+    ...     G, "x", "y", flow_func=shortest_augmenting_path
+    ... )
     True
 
     """
     if flow_func is None:
         if kwargs:
-            raise nx.NetworkXError("You have to explicitly set a flow_func if"
-                                   " you need to pass parameters via kwargs.")
+            raise nx.NetworkXError(
+                "You have to explicitly set a flow_func if"
+                " you need to pass parameters via kwargs."
+            )
         flow_func = default_flow_func
 
     if not callable(flow_func):
         raise nx.NetworkXError("flow_func has to be callable.")
 
-    if (kwargs.get('cutoff') is not None and
-        flow_func in (edmonds_karp, preflow_push, shortest_augmenting_path)):
+    if kwargs.get("cutoff") is not None and flow_func is preflow_push:
         raise nx.NetworkXError("cutoff should not be specified.")
 
-    R = flow_func(G, s, t, capacity=capacity, value_only=True, **kwargs)
+    R = flow_func(flowG, _s, _t, capacity=capacity, value_only=True, **kwargs)
 
-    return R.graph['flow_value']
+    return R.graph["flow_value"]
