@@ -47,8 +47,14 @@ With node labels:
 >>> G1 = nx.path_graph(4)
 >>> G2 = nx.path_graph(4)
 >>> mapped = {1: 1, 2: 2, 3: 3, 0: 0}
->>> nx.set_node_attributes(G1, dict(zip(G1, ["blue", "red", "green", "yellow"])), "label")
->>> nx.set_node_attributes(G2, dict(zip([mapped[u] for u in G1], ["blue", "red", "green", "yellow"])), "label")
+>>> nx.set_node_attributes(
+...     G1, dict(zip(G1, ["blue", "red", "green", "yellow"])), "label"
+... )
+>>> nx.set_node_attributes(
+...     G2,
+...     dict(zip([mapped[u] for u in G1], ["blue", "red", "green", "yellow"])),
+...     "label",
+... )
 >>> nx.vf2pp_is_isomorphic(G1, G2, node_label="label")
 True
 >>> nx.vf2pp_isomorphism(G1, G2, node_label="label")
@@ -61,6 +67,7 @@ References
    https://doi.org/10.1016/j.dam.2018.02.018
 
 """
+
 import collections
 
 import networkx as nx
@@ -97,6 +104,7 @@ _StateParameters = collections.namedtuple(
 )
 
 
+@nx._dispatchable(graphs={"G1": 0, "G2": 1}, node_attrs={"node_label": "default_label"})
 def vf2pp_isomorphism(G1, G2, node_label=None, default_label=None):
     """Return an isomorphic mapping between `G1` and `G2` if it exists.
 
@@ -127,6 +135,7 @@ def vf2pp_isomorphism(G1, G2, node_label=None, default_label=None):
         return None
 
 
+@nx._dispatchable(graphs={"G1": 0, "G2": 1}, node_attrs={"node_label": "default_label"})
 def vf2pp_is_isomorphic(G1, G2, node_label=None, default_label=None):
     """Examines whether G1 and G2 are isomorphic.
 
@@ -155,6 +164,7 @@ def vf2pp_is_isomorphic(G1, G2, node_label=None, default_label=None):
     return False
 
 
+@nx._dispatchable(graphs={"G1": 0, "G2": 1}, node_attrs={"node_label": "default_label"})
 def vf2pp_all_isomorphisms(G1, G2, node_label=None, default_label=None):
     """Yields all the possible mappings between G1 and G2.
 
@@ -473,8 +483,8 @@ def _find_candidates(
     G1, G2, G1_labels, _, _, nodes_of_G2Labels, G2_nodes_of_degree = graph_params
     mapping, reverse_mapping, _, _, _, _, _, _, T2_tilde, _ = state_params
 
-    covered_neighbors = [nbr for nbr in G1[u] if nbr in mapping]
-    if not covered_neighbors:
+    covered_nbrs = [nbr for nbr in G1[u] if nbr in mapping]
+    if not covered_nbrs:
         candidates = set(nodes_of_G2Labels[G1_labels[u]])
         candidates.intersection_update(G2_nodes_of_degree[G1_degree[u]])
         candidates.intersection_update(T2_tilde)
@@ -489,10 +499,10 @@ def _find_candidates(
             )
         return candidates
 
-    nbr1 = covered_neighbors[0]
+    nbr1 = covered_nbrs[0]
     common_nodes = set(G2[mapping[nbr1]])
 
-    for nbr1 in covered_neighbors[1:]:
+    for nbr1 in covered_nbrs[1:]:
         common_nodes.intersection_update(G2[mapping[nbr1]])
 
     common_nodes.difference_update(reverse_mapping)
