@@ -1,12 +1,12 @@
+import math
+from operator import itemgetter
+
 import pytest
 
 np = pytest.importorskip("numpy")
 
 import networkx as nx
-
-
-from networkx.algorithms.tree import branchings
-from networkx.algorithms.tree import recognition
+from networkx.algorithms.tree import branchings, recognition
 
 #
 # Explicitly discussed examples from Edmonds paper.
@@ -17,16 +17,22 @@ from networkx.algorithms.tree import recognition
 # fmt: off
 G_array = np.array([
     # 0   1   2   3   4   5   6   7   8
-    [0,  0, 12,  0, 12,  0,  0,  0,  0],  # 0
-    [4,  0,  0,  0,  0, 13,  0,  0,  0],  # 1
-    [0, 17,  0, 21,  0, 12,  0,  0,  0],  # 2
-    [5,  0,  0,  0, 17,  0, 18,  0,  0],  # 3
-    [0,  0,  0,  0,  0,  0,  0, 12,  0],  # 4
-    [0,  0,  0,  0,  0,  0, 14,  0, 12],  # 5
-    [0,  0, 21,  0,  0,  0,  0,  0, 15],  # 6
-    [0,  0,  0, 19,  0,  0, 15,  0,  0],  # 7
-    [0,  0,  0,  0,  0,  0,  0, 18,  0],  # 8
+    [0, 0, 12, 0, 12, 0, 0, 0, 0],  # 0
+    [4, 0, 0, 0, 0, 13, 0, 0, 0],  # 1
+    [0, 17, 0, 21, 0, 12, 0, 0, 0],  # 2
+    [5, 0, 0, 0, 17, 0, 18, 0, 0],  # 3
+    [0, 0, 0, 0, 0, 0, 0, 12, 0],  # 4
+    [0, 0, 0, 0, 0, 0, 14, 0, 12],  # 5
+    [0, 0, 21, 0, 0, 0, 0, 0, 15],  # 6
+    [0, 0, 0, 19, 0, 0, 15, 0, 0],  # 7
+    [0, 0, 0, 0, 0, 0, 0, 18, 0],  # 8
 ], dtype=int)
+
+# Two copies of the graph from the original paper as disconnected components
+G_big_array = np.zeros(np.array(G_array.shape) * 2, dtype=int)
+G_big_array[:G_array.shape[0], :G_array.shape[1]] = G_array
+G_big_array[G_array.shape[0]:, G_array.shape[1]:] = G_array
+
 # fmt: on
 
 
@@ -121,10 +127,12 @@ greedy_subopt_branching_1b = [
 ]
 
 
-def build_branching(edges):
+def build_branching(edges, double=False):
     G = nx.DiGraph()
     for u, v, weight in edges:
         G.add_edge(u, v, weight=weight)
+        if double:
+            G.add_edge(u + 9, v + 9, weight=weight)
     return G
 
 
@@ -143,11 +151,6 @@ def assert_equal_branchings(G1, G2, attr="weight", default=1):
     e1 = sorted_edges(G1, attr, default)
     e2 = sorted_edges(G2, attr, default)
 
-    # If we have an exception, let's see the edges.
-    print(e1)
-    print(e2)
-    print
-
     for a, b in zip(e1, e2):
         assert a[:2] == b[:2]
         np.testing.assert_almost_equal(a[2], b[2])
@@ -203,6 +206,26 @@ def test_greedy_max1():
     assert_equal_branchings(B, B_)
 
 
+def test_greedy_branching_kwarg_kind():
+    G = G1()
+    with pytest.raises(nx.NetworkXException, match="Unknown value for `kind`."):
+        B = branchings.greedy_branching(G, kind="lol")
+
+
+def test_greedy_branching_for_unsortable_nodes():
+    G = nx.DiGraph()
+    G.add_weighted_edges_from([((2, 3), 5, 1), (3, "a", 1), (2, 4, 5)])
+    edges = [(u, v, data.get("weight", 1)) for (u, v, data) in G.edges(data=True)]
+    with pytest.raises(TypeError):
+        edges.sort(key=itemgetter(2, 0, 1), reverse=True)
+    B = branchings.greedy_branching(G, kind="max").edges(data=True)
+    assert list(B) == [
+        ((2, 3), 5, {"weight": 1}),
+        (3, "a", {"weight": 1}),
+        (2, 4, {"weight": 5}),
+    ]
+
+
 def test_greedy_max2():
     # Different default weight.
     #
@@ -278,6 +301,24 @@ def test_edmonds1_maxarbor():
     assert_equal_branchings(x, x_)
 
 
+def test_edmonds1_minimal_branching():
+    # graph will have something like a minimum arborescence but no spanning one
+    G = nx.from_numpy_array(G_big_array, create_using=nx.DiGraph)
+    B = branchings.minimal_branching(G)
+    edges = [
+        (3, 0, 5),
+        (0, 2, 12),
+        (0, 4, 12),
+        (2, 5, 12),
+        (4, 7, 12),
+        (5, 8, 12),
+        (5, 6, 14),
+        (2, 1, 17),
+    ]
+    B_ = build_branching(edges, double=True)
+    assert_equal_branchings(B, B_)
+
+
 def test_edmonds2_maxbranch():
     G = G2()
     x = branchings.maximum_branching(G)
@@ -405,15 +446,13 @@ def test_edge_attribute_preservation_normal_graph():
     ]
     G.add_edges_from(edgelist)
 
-    ed = branchings.Edmonds(G)
-    B = ed.find_optimum("weight", preserve_attrs=True, seed=1)
+    B = branchings.maximum_branching(G, preserve_attrs=True)
 
     assert B[0][1]["otherattr"] == 1
     assert B[0][1]["otherattr2"] == 3
 
 
 def test_edge_attribute_preservation_multigraph():
-
     # Test that edge attributes are preserved when finding an optimum graph
     # using the Edmonds class for multigraphs.
     G = nx.MultiGraph()
@@ -425,8 +464,7 @@ def test_edge_attribute_preservation_multigraph():
     ]
     G.add_edges_from(edgelist * 2)  # Make sure we have duplicate edge paths
 
-    ed = branchings.Edmonds(G)
-    B = ed.find_optimum("weight", preserve_attrs=True)
+    B = branchings.maximum_branching(G, preserve_attrs=True)
 
     assert B[0][1][0]["otherattr"] == 1
     assert B[0][1][0]["otherattr2"] == 3
@@ -443,9 +481,144 @@ def test_edge_attribute_discard():
     ]
     G.add_edges_from(edgelist)
 
-    ed = branchings.Edmonds(G)
-    B = ed.find_optimum("weight", preserve_attrs=False)
+    B = branchings.maximum_branching(G, preserve_attrs=False)
 
     edge_dict = B[0][1]
     with pytest.raises(KeyError):
         _ = edge_dict["otherattr"]
+
+
+def test_partition_spanning_arborescence():
+    """
+    Test that we can generate minimum spanning arborescences which respect the
+    given partition.
+    """
+    G = nx.from_numpy_array(G_array, create_using=nx.DiGraph)
+    G[3][0]["partition"] = nx.EdgePartition.EXCLUDED
+    G[2][3]["partition"] = nx.EdgePartition.INCLUDED
+    G[7][3]["partition"] = nx.EdgePartition.EXCLUDED
+    G[0][2]["partition"] = nx.EdgePartition.EXCLUDED
+    G[6][2]["partition"] = nx.EdgePartition.INCLUDED
+
+    actual_edges = [
+        (0, 4, 12),
+        (1, 0, 4),
+        (1, 5, 13),
+        (2, 3, 21),
+        (4, 7, 12),
+        (5, 6, 14),
+        (5, 8, 12),
+        (6, 2, 21),
+    ]
+
+    B = branchings.minimum_spanning_arborescence(G, partition="partition")
+    assert_equal_branchings(build_branching(actual_edges), B)
+
+
+def test_arborescence_iterator_min():
+    """
+    Tests the arborescence iterator.
+
+    A brute force method found 680 arborescences in this graph.
+    This test will not verify all of them individually, but will check two
+    things
+
+    * The iterator returns 680 arborescences
+    * The weight of the arborescences is non-strictly increasing
+
+    for more information please visit
+    https://mjschwenne.github.io/2021/06/10/implementing-the-iterators.html
+    """
+    G = nx.from_numpy_array(G_array, create_using=nx.DiGraph)
+
+    arborescence_count = 0
+    arborescence_weight = -math.inf
+    for B in branchings.ArborescenceIterator(G):
+        arborescence_count += 1
+        new_arborescence_weight = B.size(weight="weight")
+        assert new_arborescence_weight >= arborescence_weight
+        arborescence_weight = new_arborescence_weight
+
+    assert arborescence_count == 680
+
+
+def test_arborescence_iterator_max():
+    """
+    Tests the arborescence iterator.
+
+    A brute force method found 680 arborescences in this graph.
+    This test will not verify all of them individually, but will check two
+    things
+
+    * The iterator returns 680 arborescences
+    * The weight of the arborescences is non-strictly decreasing
+
+    for more information please visit
+    https://mjschwenne.github.io/2021/06/10/implementing-the-iterators.html
+    """
+    G = nx.from_numpy_array(G_array, create_using=nx.DiGraph)
+
+    arborescence_count = 0
+    arborescence_weight = math.inf
+    for B in branchings.ArborescenceIterator(G, minimum=False):
+        arborescence_count += 1
+        new_arborescence_weight = B.size(weight="weight")
+        assert new_arborescence_weight <= arborescence_weight
+        arborescence_weight = new_arborescence_weight
+
+    assert arborescence_count == 680
+
+
+def test_arborescence_iterator_initial_partition():
+    """
+    Tests the arborescence iterator with three included edges and three excluded
+    in the initial partition.
+
+    A brute force method similar to the one used in the above tests found that
+    there are 16 arborescences which contain the included edges and not the
+    excluded edges.
+    """
+    G = nx.from_numpy_array(G_array, create_using=nx.DiGraph)
+    included_edges = [(1, 0), (5, 6), (8, 7)]
+    excluded_edges = [(0, 2), (3, 6), (1, 5)]
+
+    arborescence_count = 0
+    arborescence_weight = -math.inf
+    for B in branchings.ArborescenceIterator(
+        G, init_partition=(included_edges, excluded_edges)
+    ):
+        arborescence_count += 1
+        new_arborescence_weight = B.size(weight="weight")
+        assert new_arborescence_weight >= arborescence_weight
+        arborescence_weight = new_arborescence_weight
+        for e in included_edges:
+            assert e in B.edges
+        for e in excluded_edges:
+            assert e not in B.edges
+    assert arborescence_count == 16
+
+
+def test_branchings_with_default_weights():
+    """
+    Tests that various branching algorithms work on graphs without weights.
+    For more information, see issue #7279.
+    """
+    graph = nx.erdos_renyi_graph(10, p=0.2, directed=True, seed=123)
+
+    assert all(
+        "weight" not in d for (u, v, d) in graph.edges(data=True)
+    ), "test is for graphs without a weight attribute"
+
+    # Calling these functions will modify graph inplace to add weights
+    # copy the graph to avoid this.
+    nx.minimum_spanning_arborescence(graph.copy())
+    nx.maximum_spanning_arborescence(graph.copy())
+    nx.minimum_branching(graph.copy())
+    nx.maximum_branching(graph.copy())
+    nx.algorithms.tree.minimal_branching(graph.copy())
+    nx.algorithms.tree.branching_weight(graph.copy())
+    nx.algorithms.tree.greedy_branching(graph.copy())
+
+    assert all(
+        "weight" not in d for (u, v, d) in graph.edges(data=True)
+    ), "The above calls should not modify the initial graph in-place"
